/*
 * Copyright (c) 2017 Digital Bazaar, Inc. All rights reserved.
 */
'use strict';

module.exports = class DocumentCache {
  /**
   * Creates a simple document cache that retains documents for a short
   * period of time.
   *
   * FIXME: Implement simple HTTP caching instead.
   *
   * @param options the options to use:
   *          [size] the maximum size of the cache (default: 50).
   *          [expires] expiration time for each entry in ms (default: 30000).
   */
  constructor() {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        _ref$size = _ref.size;

    let size = _ref$size === undefined ? 50 : _ref$size;
    var _ref$expires = _ref.expires;
    let expires = _ref$expires === undefined ? 30000 : _ref$expires;

    this.order = [];
    this.cache = {};
    this.size = size;
    this.expires = expires;
  }

  get(url) {
    if (url in this.cache) {
      const entry = this.cache[url];
      if (entry.expires >= Date.now()) {
        return entry.ctx;
      }
      delete this.cache[url];
      this.order.splice(this.order.indexOf(url), 1);
    }
    return null;
  }

  set(url, ctx) {
    if (this.order.length === this.size) {
      delete this.cache[this.order.shift()];
    }
    this.order.push(url);
    this.cache[url] = { ctx: ctx, expires: Date.now() + this.expires };
  }
};